;_________________________________________________________________________________________________
;
; Temperature measurement with PIC16F88
; FILE NAME: AI_measure22.asm	Ver 3.0	04 May 2008
; Author   : J Vermaak
; 
; Modified code to allow for switch to let user view runtime and number of starts and stops.
; Also added heartbeat indication to indicate that the PIC did not freeze
;_________________________________________________________________________________________________
	radix	dec
	list p=16F88
	include	"P16F88.inc"

;Program Configuration Register 1
;	__CONFIG _CONFIG1, _CP_OFF & _CCP1_RB3 & _DEBUG_OFF & _WRT_PROTECT_OFF & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _MCLR_OFF & _PWRTE_ON & _WDT_OFF & _INTRC_IO

	__config _CONFIG1, 0x2F30	; sets the configuration settings (oscillator type etc.)
					; Internal RC, PowerUp Timerm, MCLR enabled
					; WRT0, WRT1
;Program Configuration Register 2
	__CONFIG _CONFIG2, _IESO_OFF & _FCMEN_OFF

; LCD PIN RW is wired to Ground		;
; NB NB NB NB NB NB NB NB NB NB		; a 10k resistor must be put between Pin5(Gnd) and Pin10 RB4 for the programming of the PIC using the
; NB NB NB NB NB NB NB NB NB NB		; PIcallW 0.16 programmer
; *************************************************************************************************
; PROGRAMMING WITH THE JUNEBUG!!!
; THE POWER TO THE CIRCUIT MUST BE ON. THE JUNEBUG FAILS IF THE POWER TO THE CIRCUIT IS NOT ON.
; *************************************************************************************************

#define	RS	PORTB,2			;
#define	E	PORTB,3			;
#define View	PORTA,0			;
#define Time	PORTA,6			; PortA<6> input for time adjustment
#define CHyst	PORTA,7			; PortA<7> input for Changing Hysteresis of Setpoint
#define	Show	Flag,0			; Variable to show elapsed time on LCD
#define	CntFlg	Flag,1			; Nuber of start Counter flag has been set
#define	QRel	Flag,2			; Flag for OUTPUT relay
Hy_05		Equ	.0050		; 05C
Hy_10		Equ	.0100		; 10C
Hy_15		Equ	.0150		; 15C
Hy_20		Equ	.0200		; 20C
Hy_25		Equ	.0250		; 25C
Hy_30		Equ	.0300		; 30C
Hy_35		Equ	.0350		; 35C
Hy_40		Equ	.0400		; 40C
Hy_45		Equ	.0450		; 45C
Hy_50		Equ	.0500		; 50C


LCD_PORT	Equ	PORTB		;
SP_C		EQU	b'01001001'	; RA1 (:18) - POT Setpoint
RM_C		EQU	b'01010001'	; RA2 (:01) - 1N4148 Room temperature
MV_C		EQU	b'01011001'	; RA3 (:02) - 1N4148 Pool Panel
Pos04		equ		0x03	; LCD Character Position 04
Pos05		equ		0x04	; LCD Character Position 05
Pos06		equ		0x05	; LCD Character Position 06
Pos07		equ		0x06	; LCD Character Position 07
Pos08		equ		0x07	; LCD Character Position 08
Pos09		equ		0x08	; LCD Character Position 09
Pos12		equ		0x0B	; LCD Character Position 12
Pos13		equ		0x0C	; LCD Character Position 13
Pos14		equ		0x0D	; LCD Character Position 14
Pos15		equ		0x0E	; LCD Character Position 15
Pos16		equ		0x0F	; LCD Character Position 16
#define Val	.3037			; See note below
; 500ms = 500000us
; Prescaler of 8 = 500000/8 = 62500 = F424.
; FFFF - F424 = 0BDB (0x0BDD = 10x3037)
; 20ms = 5000 = 1388,
; rounded = 4996 = 1384 (FFFF-1384 = EC7B = 60538)


	CBLOCK	0x20
		count			;
		count1			;
		counta			;
		countb			;
		DIGIT			;
		LCDTemp			; 4 bit for LCD
		NumH			; high byte to BCD convert
		NumL			; low byte to BCD convert
		AdcMVL			; MV Lower value
		AdcMVH			; MV Higher value
		AdcSPL			; ADC Setpoint low
		AdcSPH			; ADC Setpoint High
		SubSPL			; Subtracted Setpoint low
		SubSPH			; Subtracted Setpoint High
		s_temp			; STATUS temp register for interrupt routine
		w_temp			; W temp register for interrupt routine
		Qs			; quarter seconds (250ms)
		Rsec01			; Runtime Seconds 01
		Rsec10			; Runtime Seconds 10
		Rmin01			; Runtime Minutes 01
		Rmin10			; Runtime Minutes 10
		Rcount01		; Runtime counter 01
		Rcount10		; Runtime counter 10
		sec			; seconds
		m01			; minutes 01
		m10			; minutes 10
		h01			; hours 01
		h10			; hours 10
		Flag			; General Purpose register for flagged bits
		MessPnt			; Temp register
		HystH			; Hysteresis storage value
		HystL			; Hysteresis storage value
		EE_ADR			; EEPROM address holder	
		W_DATA			; Data register to be written to EEPROM
		R_DATA			; EEPROM Data Read
; -------------------------------------	; 
; For 24x24 multiplier			; The following registers are used
		Prod:6			; in the multiplication mathematical
		Getal:3			; sequence
		BitC:1			;
; For 24x24 divider			;
		nratorH			; and these are used fot the divider
		nratorM			; mathematical sequence
		nratorL			;
		denomH			;
		denomM			;
		denomL			;
		remainH			;
		remainM			;
		remainL			;
		shiftH			;
		shiftM			;
		shiftL			;
		BCount			;
	ENDC				;
Maaler		EQU	Prod+3		; 3 bytes shared with Product's
; ------------------------------------- ; code received from Eric Gibbs
	CBLOCK 0x70 			; ascii bfrs, these reg locations common to all BANKS	
		ASCBFR0     		; not essential that they are located in this address
		ASCBFR1     		; its easier for an extended program.
		ASCBFR2     		;
		ASCBFR3     		;
		ASCBFR4     		;
		ASCBFR5     		;
		ASCBFR6     		;
		ASCBFR7     		;
		TEMP1			; bin2asc control & cntrs   	
		TEMP2			;
		CNTR0			;
		CNTR1			;
	ENDC				;
	
Maaler		EQU	Prod+3		; 3 bytes shared with Product's
					; less significant bytes (+3..5)	
;___________________________________________________________________________________________________
		org	0x000		;
		call	Start		;
		goto	Main		;
		org 	0x004 		; dummy for interrupts
		goto	ISR		; Interrupt service routine
;______________________________________________________________________________________________
;______________________________________________________________________________________________	

HEX_Table	addwf	PCL,f		;
		retlw	'0'		;0 
		retlw	'1'		;1 
		retlw	'2'		;2 
		retlw	'3'		;3 
		retlw	'4'		;4 
		retlw	'5'		;5 
		retlw	'6'		;6 
		retlw	'7'		;7 
		retlw	'8'		;8 
		retlw	'9'		;9 

;______________________________________________________________________________________________	
SP_Table	addwf	PCL,f		;
		goto	H_05		; 05C (0)
		goto	H_10		; 10C (1)
		goto	H_15		; 15C (2)
		goto	H_20		; 20C (3)
		goto	H_25		; 25C (4)
		goto	H_30		; 30C (5)
		goto	H_35		; 35C (6)
		goto	H_40		; 40C (7)
		goto	H_45		; 45C (8)
		goto	H_50		; 50C (9)
;______________________________________________________________________________________________	
Start:					;
		bcf	PCLATH,3	; page 0
		bcf	PCLATH,4	;
		bcf	STATUS,RP0	;
		bcf	STATUS,RP1	;
		bcf	STATUS,IRP	;
		movlw	0x00		;
	  	movwf	INTCON		; disable all intr
	  	movwf	PIE1		;
		movwf	PIR1		;
		bsf	STATUS,RP0 	; Switch to Bank 1
		movlw	b'11101111' 	; Define I/O on Port A (RA5 mclr input, RA4 ADC input,
		movwf	TRISA&0x7F	; RA3, RA2 and RA1 used for AI inputs. RA4 = DI outputs, RA0,RA6,RA7 = DI)
		movlw	b'00000000'	; Define I/O on Port B for LCD display
		movwf	TRISB&0x7F	;
		movlw	0x00 		; All pins digital i/o for now
		movwf	ANSEL		;
		movlw	b'01100000' 	; Set internal osc to 4Mhz
		movwf 	OSCCON & 0x7F	;
		bcf	STATUS,RP0	; Back to Bank 0
;______________________________________________________________________	;
		clrf	TMR1L		; Clear TMR1L (lo byte) to prevent roll over
		movlw	high(Val)	; 
		movwf	TMR1H		; write value into hi_byte timer register
		movlw	low(Val)	;
		movwf	TMR1L		; write value into lo_byte timer register
		movlw	B'00110001'	; Pre-scaler set to 8, LP osc DISABLED, Start timer '0011 0001'
		movwf	T1CON		; Move into Timer1 Control Register
;______ Setup all Interrup required registers _________________________	;
		bsf	INTCON,GIE	; Enable Global interrupts
		bsf	INTCON,PEIE	; Enable Periphiral interrupt
		bsf     STATUS,RP0	; BANK 1
		bsf	PIE1,TMR1IE	; enable Timer1 Overflow interrupt
		bcf     STATUS,RP0	; BANK 0
; -------------------------------------	;
; After power failure we need to read	;
; the stored value of HYST from the	;
; EEPROM				;
; -------------------------------------	;
		clrf	EE_ADR		; Make sure EEPROM is at address zero
		call	EE_RD		; Read EEPROM Addr 0
		movwf	HystH		; Write into Hyst High register		
		incf	EE_ADR,f	; increment EE_Addr with 1
		call	EE_RD		; Read EEPROM Addr
		movwf	HystL		; Write into Hyst Low register
;______________________________________________________________________	;
		clrf	Rmin10		; Reset all the time variables
		clrf	Rmin01		; with initial startup.
		clrf	Rsec10		; 
		clrf	Rsec01		; I have found that if this is not
		clrf	Rcount10	; reset, the LCD display may get
		clrf	Rcount01	; funny data, and that can cause
		clrf	Flag		; the LCD display to hang
		clrf	sec		;
		clrf	h10		;
		clrf	h01		;
		clrf	m10		;
		clrf	m01		;
;______________________________________________________________________	;
		call	LCDInit		; Initialize the LCD Display	
		call 	ADC_Init	; once only
		return			;
;___________________________________________________________________________________________________
Main					;
		call	MV_Temp		; Display MV (Pool temp) on LCD line 1
		call	RM_Temp		; Display Room temperature on LCD Line
		call 	Compare1	; Test if MV > SP
		call	SP_Temp		; Display Setpoint value on line 1
		call	HB		;
		call	Del100		; Delay for 100 us
		call	TimePB		; Test if time PB is pressed?
		call	HystPB		; Test if the Hysteresis PB was pressed?
		call	TestOP		;
		call	WysTyd		;
		goto	Main		; loop for ever

;_____________________________________________________________________________________________________
TestOP:					; Allows The Flag<bit> to be simulated
		btfss	QRel		;
		bcf	PORTA,4		; Switch OFF the output relay
		btfsc	QRel		;
		bsf	PORTA,4		; Switch ON the output relay
		return			;
;_____________________________________________________________________________________________________
; RA3 Temp. Routine to show the first temperature sensor on LCD line 1. Sensor = 1N4148 silicon diode
;_____________________________________________________________________________________________________
MV_Temp:				; MV Value
		call	LCD_L1P01	; set to first line Pos 1
		movlw	MV_C		; RA3 1N4148 sensor
		movwf	ADCON0		; Put RA2 address in ADCON0
		call	READ_AD		; Read AD2. Store results in NumH, NumL
		call	Multiply_24x24	; Multiply the Result with 1000 dec
		call	Divide_24x24	; Now divide the answer with 2046 dec
					; result in registers nratorH & nratorM
		movf    nratorL,w	; Store the MV values into registers
		movwf   AdcMVL		; Save MV Low value
		movf    nratorM,w	; and ADcSH
		movwf   AdcMVH		; Save MV high value
		call	Bin2Asc		; Convert Binary value into ASCII
; -------------------------------------	;
; Test if Elapsed time should be 	;
; displayed on LCD for 10 seconds	;
; -------------------------------------	;
		btfsc	Show		; Test if Show is set?
		return			; Return to Main
		call	Data2LCD	; Write the values to LCD
		movlw	0xDF		; Shows ""
		call	LCDChar		; display decimal point
		movlw	'C'		; Shows "C"
		call	LCDChar		; display decimal point
		movlw	' '		;
		call	LCDChar		; display decimal point
		movlw	'M'		;
		call	LCDChar		; display decimal point
		movlw	'V'		;
		call	LCDChar		; display decimal point
		movlw	' '		;
		call	LCDChar		; display decimal point
		return			; Return to Main
;_____________________________________________________________________________________________________
; RA0 Temp. Routine to show the temperature control setpoint on LCD line 2 bottom right
;_____________________________________________________________________________________________________
SP_Temp:				; SETPOINT Value
		call	LCD_L1P12	; LCD Line 1 Pos 11
		movlw	SP_C		;
		movwf	ADCON0		; Put RA1 address in ADCON0
		call	READ_AD		; Read AD channel. Store results in NumH, NumL
		call	Multiply_24x24	; Multiply the Result with 1000 dec
		call	Divide_24x24	; Now divide the answer with 2046 dec
					; result in registers nratorH & nratorM
		movf    nratorL,w	; Store the MV values into registers
		movwf   AdcSPL		; Save SP Low value
		movf    nratorM,w	; and ADcSH
		movwf   AdcSPH		; Save SP high value
		call	Bin2Asc		; Convert Binary value into ASCII
; -------------------------------------	;
; Test if Elapsed time should be 	;
; displayed on LCD for 10 seconds	;
; -------------------------------------	;
		btfsc	Show		; Test if Show is set?
		return			; Return to Main
		call	Data2LCDS	; Write the values to LCD
		movlw	0xDF		; Shows ""
		call	LCDChar		; display decimal point
		movlw	'S'		;
		call	LCDChar		; display decimal point
		movlw	'P'		;
		call	LCDChar		; display decimal point
		return			; Return to Main
; -------------------------------------	;
; Heartbeat
; -------------------------------------	;
HB:					;
		movlw	Pos16		; This code will put the LCD cursor at Pos05
		call	LCD_Line1W	; Line 1 LCD
		btfsc	Show		; test if the Carry bit is set
		goto	HBOn		; N
		goto	HBOff		; Y
HBOn:					; Heart indication
		movlw	' '		;
		call	LCDChar		; display decimal point
		return			;
HBOff:					; Heart indication
		movlw	'*'		;
		call	LCDChar		; display decimal point
		return			;
;_____________________________________________________________________________________________________
; RA2 Temp. Routine to show the second temperature sensor on LCD line 2. Sensor = 1N4148 
;_____________________________________________________________________________________________________
RM_Temp:				; Room temperature
		call	LCD_L2P01	; set to first line
		movlw	RM_C		; 1N4148 room temperature measurement
		movwf	ADCON0		; Put RA3 address in ADCON0
		call	READ_AD		; Read AD channel. Store results in NumH, NumL
		call	Multiply_24x24	; Multiply the Result with 1000 dec
		call	Divide_24x24	; Now divide the answer with 2046 dec
					; result in registers nratorH & nratorM
		call	Bin2Asc		; Convert Binary value into ASCII
; -------------------------------------	;
; Test if Elapsed time should be 	;
; displayed on LCD for 10 seconds	;
; -------------------------------------	;
		btfsc	Show		; Test if Show is set?
		return			; Return to Main
		call	Data2LCD	; Write the values to LCD
		movlw	0xDF		; Shows ""
		call	LCDChar		; display decimal point
		movlw	'C'		; Shows "C"
		call	LCDChar		; display decimal point
		movlw	' '		;
		call	LCDChar		; display decimal point
		movlw	'I'		;
		call	LCDChar		; display decimal point
		movlw	'n'		;
		call	LCDChar		; display decimal point
		movlw	' '		;
		call	LCDChar		; display decimal point
		return			; Return to Main
;_____________________________________________________________________________________________________
;_____________________________________________________________________________________________________
READ_AD:
		bsf	ADCON0,ADON 		; Turn the ADC back on
		call	Del100			;
		bsf	ADCON0,GO_DONE		; initiate conversion
		btfsc	ADCON0,GO_DONE		; Test if done yet
		goto	$-1			; wait for ADC to finish
; ---------------------------------------------	;
		nop	; Now you need to simulate the			
		nop	; ADRESH and ADRESL values
		nop	; for testing purpose
; ---------------------------------------------	;
		movf	ADRESH,W		; Move high value result intp NumH
		andlw	0x03			; Max allowable value that can be written into NumH
		movwf	NumH			; is 0x03 (EG = AdcVH)
		bsf	STATUS,RP0 		; Bank 1
		movf	ADRESL,W		; NumL
		bcf	STATUS,RP0 		; Bank 0
		movwf	NumL			; return result in NumL and NumH (EG = AdcVL)
		return				;
;___________________________________________________________________________________________________
;
; ADC Init Need to run once only
;___________________________________________________________________________________________________
ADC_Init:
; --------------------------------------------- ;
; Register ANSEL 
; This register is used to indicate which AI channels will be used
; Every channel marked with a '1' will be used as an AI signal.
; The corresponding channel MUST also be configured as as input
; signal with TRISA
; --------------------------------------------- ;
		bsf	STATUS,RP0 		; Select BANK 1
		movlw	b'00001110'		; ANSEL
;--------------		  |||||||+-- - - - - - -; ANS0 (RA0) not used
;			  ||||||+--- - - - - - -; ANS1 (RA1) USED
;			  |||||+---- - - - - - -; ANS2 (RA2) USED
;			  ||||+----- - - - - - -; ANS3 (RA3) USED
;			  |||+------ - - - - - -; ANS4 (RA4) not used
;			  ||+------- - - - - - -; ANS5 (RB6) not used
;			  |+-------- - - - - - -; ANS6 (RB7) not used
;			  +--------- - - - - - -; Unused
		movwf	ANSEL			; 
; --------------------------------------------- ;
; Register ADCON1
; --------------------------------------------- ;
; Register ADCON1
; This register is used to setup voltage reference for input
; measurement. This is used to setup the justification of the result (left/right)
; BITS <3:0> are unused in
; ---------------------------------------------	;
; BITS <5:4> are used for voltage reference setup
; VCFG	5:4	-	Vref+		Vref- 
;	0,0	-	AVdd(14)	AVss(5)
;	0,1	-	AVdd(14)	VRef- (RA2(1))
;	1,0	-	VRef+(RA3(2))	AVss(5)
;	1,1	-	VRef+(RA3(2))	VRef- (RA2(1))
; ---------------------------------------------	;
; BIT <6> AD Clock source by 2 divide select	
; BIT <7> AD result format select
;      1	-	Right justified (ADRESH 000000xx)
;      0	-	Left justified  (ADRESH xx000000)	
; --------------------------------------------- ;
		movlw	b'11000000'		; ADCON1 '11100000'
;--------------		  |||||||+-- - - - - - -; Unused
;			  ||||||+--- - - - - - -; Unused
;			  |||||+---- - - - - - -; Unused
;			  ||||+----- - - - - - -; Unused
;			  |||+------ - - - - - -; VCFG0 (Vref-) (00 = Vdd, Vss,
;			  ||+------- - - - - - -; VCFG1 (Vref+)
;			  |+-------- - - - - - -; ADCS2 (Clock divide by 2 select)
;			  +--------- - - - - - -; ADFM (0=left, 1=right justification)
		movwf	ADCON1&0x7F	  	;
;-----------------------------------------------;
		bsf	TRISA&0x7F,1		; AN1 is the setpoint POT input
		bsf	TRISA&0x7F,2 		; Make RA2 input Temp Sensor via 1N4148
		bsf	TRISA&0x7F,3 		; Make RA3 input Temp sensor via AD592
		bcf	STATUS,RP0 		; Select BANK 0
; --------------------------------------------- ;
		return				;

; Register ADCON0
; This register handles the convertion clock frequency setup,
; decides which AD channel is being converted and indicated when AD 
; convertion is complete
; BITS <7:6> AD convertion clock select
;       0,0	-	FOSC/2
;	0,1	-	FOSC/8
;	1,0	-	FOSC/32
;	1,1	-	Frc - Internal RC oscillator)
; BITS <5:4:3> AD Channel selection bits
;	0,0,0	-	RA0
;	0,0,1	-	RA1
;	0,1,0	-	RA2 (1N4148)
;	0,1,1	-	RA3 (AD592)
;	1,0,0	-	RA4
;	1,0,1	-	RA5
;	1,1,0	-	RA6
; BIT  <2> GO/Done
;	0	-	AD conversion complete
;	1	-	AD conversion in progress
; BIT  <1> Not used Always  0
; BIT  <0>
;	0	-	AD converter module OFF
;	1	-	AD converter module ON
; --------------------------------------------- ;
;		movlw	b'01010001'		; ADCON0
;--------------		  |||||||+-- - - - - - -; ADON (1 = AD on, O = AD off)
; 			  ||||||+--- - - - - - -; Unused
; 			  |||||+---- - - - - - -; GO/DONE
;			  ||||+----- - - - - - -; CHS0 -\ (RA2 = 010
;			  |||+------ - - - - - -; CHS1 - = Analog channel select
;			  ||+------- - - - - - -; CHS2 -/
;			  |+-------- - - - - - -; ADCS0 - AD convertion clock select
;			  +--------- - - - - - -; ADCS1
;		movwf	ADCON0		  	;
;-----------------------------------------------;
;		return
;_____________________________________________________________________________________________________		
; Bin2Asc code received from Eric Gibbs
Bin2Asc:
	;clrf	ASCBFR0 ;left for ref only
	;clrf	ASCBFR1
	;clrf	ASCBFR2
	;clrf	ASCBFR3
	;clrf	ASCBFR4
	clrf	ASCBFR5
	clrf	ASCBFR6
	clrf	ASCBFR7
	movlw	0x0
	movwf	nratorH
    	movlw	0X18				; 24 bits,,,use 18 if 16 bits to do, else 24bit
	movwf	TEMP1

BITLP:						; shift msb into carry
	rlf	nratorL,F			; nratorL,w
	rlf	nratorM,F			; nratorM,w
	rlf	nratorH,F			; nratorH,w	
	movlw	ASCBFR7				; 0=msd 1st
	movwf	FSR				; fsr=pointer to digits
	movlw	0X8				; digits to do
	movwf	TEMP2
ADJLP:
	rlf	INDF,F				; shift digit 1 bit left
	movlw	0x0A
	subwf	INDF,W				; check and adjust for decimal overflow
	btfsc	STATUS,C
	movwf	INDF
	decf	FSR,F				;incf=msd 1st next digit
	decfsz	TEMP2,F
	goto	ADJLP
	decfsz	TEMP1,F				;next bit
	goto	BITLP
	movlw	0X30				; make asci
	;iorwf	ASCBFR0,F
	;iorwf	ASCBFR1,F
	;iorwf	ASCBFR2,F
	;iorwf	ASCBFR3,F
	;iorwf	ASCBFR4,F
	iorwf	ASCBFR5,F			; only 3 lsd used in this program
	iorwf	ASCBFR6,F
	iorwf	ASCBFR7,F
	return		
;_____________________________________________________________________________________________________
Data2LCD:
	movf	ASCBFR5,W		;
	call	LCDChar			;
	movf	ASCBFR6,W		;
	call	LCDChar			;
	movlw	'.'			;
	call	LCDChar			;
	movf	ASCBFR7,W		;
	call	LCDChar			;
	return				;
Data2LCDS:				;
	movf	ASCBFR5,W		;
	call	LCDChar			;
	movf	ASCBFR6,W		;
	call	LCDChar			;
	return
;_____________________________________________________________________________________________________
; Compare code received from Eric Gibbs
Compare1: 
		call	LCD_L2P11	;
		btfss	QRel		; Test if Output is set
		goto	Cntrl		; (0)
; Subtract the Hysterisis from Setpoint	;
;		movlw	high(Hyst)	; Upper Nibble of Hyst
		movf	HystH,w		;
		subwf	AdcSPH,w	; a + 1
		movwf	SubSPH		; c + 1
;		movlw	low(Hyst)	; Lower nibble of Hyst
		movf	HystL,w		;
		subwf	AdcSPL,w	; 	
		movwf	SubSPL		;
		btfss	STATUS,C	; Look for the Carry
		decf	SubSPH		;
; Move modified setpoint into original registers
		movf	SubSPL,w	; move SubSPL -> w
		movwf	AdcSPL		;
		movf	SubSPH,w	; move SubSPH -> w
		movwf	AdcSPH		;
; ------------------------------------- ; 	
Cntrl:		movf	AdcMVL,W 	; MV low value
		subwf	AdcSPL,W 	; SetPoint lower value
		movf	AdcMVH,W 	; MV High value
		btfss	STATUS,C	;
		addlw	0x01		;
		subwf	AdcSPH,W 	; SetPoint High value
		btfsc	STATUS,C	; AdcRH,7;neg number
		goto	Lower1		;
; ------------------------------------- ; 	
Higher1:				; the PUMP label could be written once.!
		bsf 	QRel		; Pump control bit ON
		btfss	CntFlg		; Test if Counter flag is set?
		incf	Rcount01,f	;
		bsf	CntFlg		; Set counter flag
		movf	Rcount01,w	; now test if Rcount01 > 10
		sublw	.10		; subtract 10 from sec
		btfss	STATUS,Z	; test if the ZERO bit is set
		goto	CntEnd		; 0
		clrf	Rcount01	; 1
;______ Seconds10 _____________		;
		incf	Rcount10,f	; Increment sec10 with 1
		movf	Rcount10,w	; now test if sec > 6
		sublw	.10		; subtract 10 from sec
		btfss	STATUS,Z	; test if the ZERO bit is set
		goto	CntEnd		; 0
		clrf	Rcount10	; 1
CntEnd:					;
; -------------------------------------	;
; Test if Elapsed time should be 	;
; displayed on LCD for 10 seconds	;
; -------------------------------------	;
		btfsc	Show		; Test if Show is set?
		return			; Return
		movlw	'P'		;
		call	LCDChar		;
		movlw	'P'		;
		call	LCDChar		;
		movlw	' '		;
		call	LCDChar		;
		movlw	'O'		;
		call	LCDChar		;
		movlw	'n'		;
		call	LCDChar		;
		movlw	' '		;
		call	LCDChar		;
		return			;
Lower1:					;
		bcf 	QRel		; Pump control bit  OFF
		bcf	CntFlg		; ReSet counter flag
; -------------------------------------	;
; Test if Elapsed time should be 	;
; displayed on LCD for 10 seconds	;
; -------------------------------------	;
		btfsc	Show		; Test if Show is set?
		return			; Return
		movlw	'P'		;
		call	LCDChar		;		
		movlw	'P'		;
		call	LCDChar		;
		movlw	' '		;
		call	LCDChar		;
		movlw	'O'		;
		call	LCDChar		;		
		movlw	'f'		;
		call    LCDChar		;
		movlw	'f'		;
		call	LCDChar		;		
		return			;
;_____________________________________________________________________________________________________
; Multiply 24x24
;_____________________________________________________________________________________________________
Multiply_24x24
; This routine multiplies the results of NumH/NumL with decimal 1000
;
		clrf	Getal			;
		movf	NumH,w			; Get H value form A/D reader
		movwf	Getal+1			; Put into Mid range register
		movf	NumL,w			; Get L value form A/D reader
		movwf	Getal+2			; Put into Low range register		
		movlw	0x00			; MSB (Most)
		movwf	Maaler			;
		movlw	0x03			; 0x03e8 = 1000 dec
		movwf	Maaler+1		;
		movlw	0xe8			;
		movwf	Maaler+2		;
		clrf	Prod			; clear destination
		clrf	Prod+1			;
		clrf	Prod+2			;
		movlw	d'24'			;
		movwf	BitC			; number of bits
		rrf	Prod+3,f		; shift out to carry
		rrf	Prod+4,f		; next multiplier bit
		rrf	Prod+5,F		;
ADD_LOOP_24x24
		btfss	STATUS,C		; if carry is set we must add multipland
						; to the product
		goto	SKIP_LOOP_24x24		; nope, skip this bit
		movf	Getal+2,W		; get LSB of multiplicand
		addwf	Prod+2,F		; add it to the lsb of the product
		movf	Getal+1,W		; middle byte
		btfsc	STATUS,C		; check carry for overflow
		incfsz	Getal+1,W		; if carry set we add one to the source 
		addwf	Prod+1,F		; and add it  (if not zero, in
						; that case mulitpland = 0xff->0x00 )
		movf	Getal,W			; MSB byte
		btfsc	STATUS,C		; check carry
		incfsz	Getal,W			;
		addwf	Prod,F			; handle overflow
		
SKIP_LOOP_24x24
        ; note carry contains most significant bit of
        ; addition here
        ; shift in carry and shift out
        ; next multiplier bit, starting from less
        ; significant bit
		rrf	Prod,F			;		
		rrf	Prod+1,f		;
		rrf	Prod+2,f		;
		rrf	Prod+3,f		;
		rrf	Prod+4,f		;
		rrf	Prod+5,F		;
		decfsz	BitC,F			;
		goto	ADD_LOOP_24x24		;	
		return				;
; End Multiply 24x24  -----------------------------------------------
;_____________________________________________________________________________________________________
; Divide_24x24 
;_____________________________________________________________________________________________________
;--------------------------------------------------------------------
; SUBROUTINE - 24 BIT DIVISION  from  TONY NIXON
;--------------------------------------------------------------------
; numerator / denominator = result + remainder
;
; numerator:        nratorH  nratorM  nratorL (Value to be divided)
; denominator:      denomH   denomM   denomL (Value dividing)
;
; result:           nratorH  nratorM  nratorL
; remainder:        remainH  remainM  remainL
;
Divide_24x24:
; This routine will divide the result from the multipier routine
; and divide it with 2046 for a range of 0..50C or
; divide by 1023 for a range of 0..100C

		movf	Maaler,w	; Get value from multiplier
		movwf	nratorH		;
		movf	Maaler+1,w	;
		movwf	nratorM		;
		movf	Maaler+2,w	;
		movwf	nratorL		;				
;______________________________________	;
; Remove the divide factor not used	;
;______________________________________	;
; Divide by 2046 for a range of 0-50C	;
;		movlw	0x00		; Dividing (denominator) = 2046 dec
;		movwf	denomH		; 2046 = 0x07FE
;		movlw	0x07		;
;		movwf	denomM		;
;		movlw	0xFE		;
;		movwf	denomL		;
;--------------------------------------	;
; Divide by 1023 for a range of 0-100C	;
		movlw	0x00		; Dividing (denominator) = 1023 dec
		movwf	denomH		; 1023 = 0x03FF
		movlw	0x03		;
		movwf	denomM		;
		movlw	0xFF		;
		movwf	denomL		;		
; -------------------------------------	;
; Code starts here			;
; -------------------------------------	;
		movlw	.24		; set decimal 24 loop count
		movwf	BCount		;
		movf	nratorH,w	; copy Numerator into Shift Holding ram registers
		movwf	shiftH		;
		movf	nratorM,w	;
		movwf	shiftM		;
		movf	nratorL,w	;
		movwf	shiftL		;
		clrf	nratorH		;  clear final Answer Numerator Ram locations
		clrf	nratorM		;
		clrf	nratorL		;
		clrf	remainH		; clear final Answer Remainder Ram locations
		clrf	remainM		;
		clrf	remainL		;
dloip:					;
		bcf	STATUS,C	; bit clear Carry Flag in STATUS register
		rlf	shiftL,f	; Shift numerator(dividend) Left to move
		rlf	shiftM,f	; next bit to remainder
		rlf	shiftH,f	; and shift in next bit of result
		rlf	remainL,f	; shift carry (next Dividend bit) into remainder
		rlf	remainM,f	;
		rlf	remainH,f	;
		movf	denomH,w	;
		subwf	remainH,w  	; subtract divsor(denomH) from(newly shifted left) Remainder HIGH byte.
		btfss	STATUS,Z	;
		goto	nichk		; skip if result was ZERO from good subtraction result
		movf	denomM,w	;
		subwf	remainM,w	; subtract divsor(denomM) from(newly shifted left) Remainder MIDDLE byte.
		btfss	STATUS, Z	;
		goto	nichk		; skip if result was ZERO from good subtraction result
		movf	denomL,w	;
		subwf	remainL,w	; subtract divsor(denomL) from(newly shifted left) Remainder LOW byte.
nichk:					;
		btfss	STATUS,C	; Carry SET? then denom is larger than reemainder
		goto	nogi		;
		movf	denomL,w	;
		subwf	remainL,f	; Subtract denominator from remainder value in Low Byte
		btfsc	STATUS,C	; Carry Set? Then execute fixup code for when a borrow is generated
		goto	nodec_remainM	; when no borrow bit is needed from the higher byte positions.
		decf	remainM,f	; Decrement to Borrow from Middle Byte, because carry was SET.
		movf	remainM,w	;
		xorlw	0xff		; Check if rollover from Borrow occurred. remainM value went from 0 to 0xFF 
		btfsc	STATUS,Z	;
		decf	remainH,f	; ZERO bit set, yes rollover, so Decrement to Borrow from High Byte, too!
nodec_remainM:				;
		movf	denomM,w	;
		subwf	remainM,f	; Subtract denominator from remainder value in Middle Byte
		btfss	STATUS,C	;
		decf	remainH,f	; Decrement High Byte, to borrow 1 bit
		movf	denomH,w	;
		subwf	remainH,f	; Subtract denominator from remainder value in High Byte
		bsf	STATUS,C	; set CARRY bit to rotate in Numerator Result next.
nogi:					;
		rlf	nratorL,f	; rotate Numerator result left 1 bit
		rlf	nratorM,f	;
		rlf	nratorH,f	;
		decfsz	BCount,f	; decrement the Loop Bit Counter
		goto	dloip		;
      					;
		return			; all done
; END 24x24 Diveder REV2
;_____________________________________________________________________________________________________
; This routine does the LCD initialisation but also does the initialisation for the Proteus simulation.
;_____________________________________________________________________________________________________
LCDInit						;  Do the 4 Bit Initialization
						;
		call	Del05			;  Wait 15 msecs
		call	Del05			;
		call	Del05			;
		movlw	0x030			;  Send the Reset Instruction
		movwf	LCD_PORT		;
		call	Pulse_e			; Pulse E
		call	Del05			; Delay 5ms
		call	Pulse_e			; Pulse E
		call	D160us			; Delay of 160us
		call	Pulse_e			; Pulse E
		call	D160us			; Delay of 160us
		movlw	0x020			; Send the Data Length Specification
		movwf	LCD_PORT		;
		call	Pulse_e			; Pulse E
		call	D160us			; Delay of 160us
		movlw	0x028			; Set Interface Length
		call	LCDIns			;
		movlw	0x010			; Turn Off Display
		call	LCDIns			; 
		movlw	0x001			; Clear Display RAM
		call	LCDIns			;
		movlw	0x006			; Set Cursor Movement
		call	LCDIns			;
		movlw	0x00C			; Turn on Display/Cursor
		call	LCDIns			;
		return				;
;_____________________________________________________________________________________________________
;_____________________________________________________________________________________________________
LCDIns						; Send the Instruction to the LCD
		movwf	LCDTemp			; Save the Value
		andlw	0xF0			; Most Significant Nybble first
		movwf	LCD_PORT		;
		bcf	RS			;
		call	Pulse_e			;
		swapf	LCDTemp, w		; Least Significant Nybble Second
		andlw	0xF0			;
		movwf	LCD_PORT		;
		bcf	RS			;
		call	Pulse_e			;
		call	Del01			; wait 1 ms
		movf	LCDTemp, w		;
		andlw	0xFC			; Have to Delay 5 msecs?
		btfsc	STATUS, Z		;
		call	Del01			; 1ms
		return				;
;_____________________________________________________________________________________________________
LCD_CharD	addlw	0x30			; add 0x30 to convert to ASCII
LCDChar						; Send the Character to the LCD
		movwf	LCDTemp			; Save the Value
		andlw	0xF0			; Most Significant Nybble first
		movwf	LCD_PORT		;
		bsf	RS			;	
		call	Pulse_e			;
		swapf	LCDTemp, w		; Least Significant Nybble Second
		andlw	0xF0			;
		movwf	LCD_PORT		;
		bsf	RS			;
		call	Pulse_e			;
		call	Del05			;
		nop				;
		return				;
;___________________________________________________________________________________________________
LCD_L1P01:	movlw	0x80			; move to 1st row, first column
		call	LCDIns			;
		retlw	0x00			;
;___________________________________________________________________________________________________
LCD_L1P12:	movlw	0x8A			; move to 1st row, 11th column
		call	LCDIns			;
		retlw	0x00			;
;___________________________________________________________________________________________________
LCD_L2P01:	movlw	0xc0			; move to 2nd row, first column
		call	LCDIns			;
		retlw	0x00			;
;___________________________________________________________________________________________________
LCD_L2P11:	movlw	0xca			; move to 2nd row, first column
		call	LCDIns			;
		retlw	0x00			;
;___________________________________________________________________________________________________
LCD_Clr		movlw	0x01			; Clear display
		call	LCDIns			;
		retlw	0x00			;
;___________________________________________________________________________________________________
Pulse_e						;
		bsf	E			; LCD Enable pulse to write data from PORTB
		nop				; into LCD module.
		bcf	E			; 
		nop				;
		retlw	0x00			;
;___________________________________________________________________________________________________
LCD_Line1W	addlw	0x80		; move to 1st row, column W
		call	LCDIns		;
		retlw	0x00		;
;___________________________________________________________________________________________________
LCD_Line2W	addlw	0xc0		; move to 2nd row, column W
		call	LCDIns		;
		retlw	0x00		;
;_____________________________________________________________________________________________________
; Routine to clear LCD of all temperature and Setpoint values and show the number of times the pump has
; started, and the accumilated elapsed time the pump has been in operation
;_____________________________________________________________________________________________________
WysTyd:					;
		btfss	View		; switch closed? (Logical 1 = On/Pressed)
		return			; No, back to main loop
		call	Del50		; Make sure the switch was pressed, and not some bogus bug
		btfsc	View		; switch closed? (Logical 1 = On/Pressed)
		goto	$ - 1		; No, Keep on cheking until released
		call	Del50		; Make sure the switch was pressed, and not some bogus bug
		btfsc	View		; Check if the switch was released
		goto	$ - 5		; No, Wait 5ms again
; -------------------------------------	;
; switch was pressed long enough	;
; -------------------------------------	;
		call	LCD_Clr		; Clear LCD display
; -------------------------------------	;
; Put "Runtime" on Line 1		;
; -------------------------------------	;
		movlw	'R'		; R
		call	LCDChar		;		
		movlw	'u'		; u
		call	LCDChar		;
		movlw	'n'		; n
		call	LCDChar		;
		movlw	't'		; t
		call	LCDChar		;		
		movlw	'i'		; i
		call    LCDChar		;
		movlw	'm'		; m
		call	LCDChar		; 
		movlw	'e'		; e
		call	LCDChar		;
		movlw	' '		;  
		call	LCDChar		;
		movlw	' '		;  
		call	LCDChar		;
		movlw	' '		;  
		call	LCDChar		;
		movlw	' '		;  
		call	LCDChar		;
		movfw	Rmin10		; move m10 to w to be displayed
		call	HEX_Table	;
		call	LCDChar		;
		movfw	Rmin01		; move m10 to w to be displayed
		call	HEX_Table	;
		call	LCDChar		;
		movlw	':'		;
		call	LCDChar		;
		movfw	Rsec10		; move m10 to w to be displayed
		call	HEX_Table	;
		call	LCDChar		;
		movfw	Rsec01		; move m10 to w to be displayed
		call	HEX_Table	;
		call	LCDChar		;
; -------------------------------------	;
; Put "Start Counter " on Line 2	;
; -------------------------------------	;
		call	LCD_L2P01	; Line 2
		movlw	'S'		; S
		call	LCDChar		;		
		movlw	't'		; t
		call	LCDChar		;
		movlw	'a'		; a
		call	LCDChar		;
		movlw	'r'		; r
		call	LCDChar		;		
		movlw	't'		; t
		call    LCDChar		;
		movlw	' '		;  
		call	LCDChar		; 
		movlw	'C'		; C
		call	LCDChar		;
		movlw	'o'		; o
		call	LCDChar		;
		movlw	'u'		; u
		call	LCDChar		;
		movlw	'n'		; n
		call	LCDChar		;
		movlw	't'		; t
		call	LCDChar		;
		movlw	'e'		; e
		call	LCDChar		;
		movlw	'r'		; r
		call	LCDChar		;
		movlw	' '		;
		call	LCDChar		;
		movfw	Rcount10	; 
		call	HEX_Table	;
		call	LCDChar		;
		movfw	Rcount01	; 
		call	HEX_Table	;
		call	LCDChar		;
		call	TwoSec		;
;_____________________________________________________________________________________________________
; This routine will show the real time clock on LCD for the last three seconds of the minute.
;_____________________________________________________________________________________________________
ShowTime:				;
		call	LCD_L1P01	; Line 1 LCD
		movlw	'T'		; T
		call	LCDChar		;
		movlw	'i'		; i
		call	LCDChar		;
		movlw	'm'		; m
		call	LCDChar		;
		movlw	'e'		; e
		call	LCDChar		;
		movlw	' '		;  
		call	LCDChar		;
		movlw	'='		; =
		call	LCDChar		;
		movlw	' '		;  
		call	LCDChar		;
		movfw	h10		; Get H10 value
		call	HEX_Table	;
		call	LCDChar		;
		movfw	h01		; Get H01 value
		call	HEX_Table	;
		call	LCDChar		;
		movlw	':'		; :
		call	LCDChar		;
		movfw	m10		; Get H10 value
		call	HEX_Table	;
		call	LCDChar		;
		movfw	m01		; Get H01 value
		call	HEX_Table	;
		call	LCDChar		;
ShowHystSetting:			;
		call	LCD_L2P01	; Line 1 LCD
		movlw	'H'		; H
		call	LCDChar		;
		movlw	'y'		; y
		call	LCDChar		;
		movlw	's'		; s
		call	LCDChar		;
		movlw	't'		; t
		call	LCDChar		;
		movlw	' '		;  
		call	LCDChar		;
		movlw	'='		; =
		call	LCDChar		;
		movlw	' '		;  
		call	LCDChar		;
		movlw	'S'		; S
		call	LCDChar		;
		movlw	'P'		; P
		call	LCDChar		;
		movlw	' '		;  
		call	LCDChar		;
		movlw	'-'		; -
		call	LCDChar		;
		movlw	' '		;  
		call	LCDChar		;
; -------------------------------------	;
; Call the values stored in HystL and	;
; HystH and put them into nratorL and	;
; nratorM, becaues the Bin2Asc call	;
; uses those two variables		;
; -------------------------------------	;
		clrf	nratorH		;
		movf    HystL,w		; Store the MV values into registers
		movwf   nratorL		; Save MV Low value
		movf    HystH,w		; and ADcSH
		movwf   nratorM		; Save MV high value
		call	Bin2Asc		; Convert Binary value into ASCII
		call	Data2LCD	; Write the values to LCD
		call	TwoSec		;
		return			;
;____________________________________________________________	
; This routine allows the user to change the real time clock
;____________________________________________________________	
TimePB					;
		btfss	Time		; switch closed? (Logical 1 = On/Pressed)
		return			; No, back to main loop
		call	Del50		; Make sure the switch was pressed, and not some bogus bug
		btfsc	Time		; switch closed? (Logical 1 = On/Pressed)
		goto	$ - 1		; No, Keep on cheking until released
		call	Del50		; Make sure the switch was pressed, and not some bogus bug
		btfsc	Time		; Check if the switch was released
		goto	$ - 5		; No, Wait 5ms again
; -------------------------------------	;
		call	LCD_Clr		; Clear
		call	LCD_L2P01	; Cursor at beginning of line 2
		movlw	'E'		; 
		call	LCDChar		;
		movlw	'n'		; 
		call	LCDChar		;
		movlw	't'		; 
		call	LCDChar		;
		movlw	'e'		; 
		call	LCDChar		;
		movlw	'r'		; 
		call	LCDChar		;
		movlw	' '		; 
		call	LCDChar		;
		movlw	'T'		; 
		call	LCDChar		;
		movlw	'i'		; 
		call	LCDChar		;
		movlw	'm'		; 
		call	LCDChar		;
		movlw	'e'		; 
		call	LCDChar		;
		movlw	' '		; 
		call	LCDChar		;
;/************************************\	;
; Hours 10's				;
;\************************************/	;
		movlw	'H'		; H
		call	LCDChar		;
		movlw	'1'		; 1
		call	LCDChar		;
		movlw	'0'		; 0
		call	LCDChar		;
		movlw	'='		; =
		call	LCDChar		;
		movlw	'?'		; ?
		call	LCDChar		;
		btfsc	Time		; Wait for the user to release the switch button
		goto	$ - 1		; the sequence will loop forever
		call	Del50		; Make sure the switch was pressed, and not some bogus bug
		call	SW_Ti		; Wait for data to enter
		movlw	Pos05		; This code will put the LCD cursor at Pos05
		call	LCD_Line1W	; Line 1 LCD
		movf	DIGIT,W		; Move value of DIGIT into W
		movwf  	h10		; Move W into HOUR10
		call	HEX_Table	; Routine calling data from a DT table
		call	LCDChar		;
;/************************************\	;
; Hours 01's				;
;\************************************/	;
		movlw	Pos12		; This code will put the LCD cursor at Pos12
		call	LCD_Line2W	; Line 2 LCD
		movlw	'H'		; H
		call	LCDChar		;
		movlw	'0'		; 0
		call	LCDChar		;
		movlw	'1'		; 1
		call	LCDChar		;
		movlw	'='		; =
		call	LCDChar		;
		movlw	'?'		; ?
		call	SW_Ti		; Wait for data to enter
		movlw	Pos06		; This code will put the LCD cursor at Pos06
		call	LCD_Line1W	; Line 1 LCD
		movf	DIGIT,W		;
		movwf  	h01		;
		call	HEX_Table	; Routine calling data from a DT table
		call	LCDChar		; Write the character data to the LCD
		movlw   ':'		;
		call	LCDChar		;
;/************************************\	;
; Minutes 10's				;
;\************************************/	;
		movlw	Pos12		; This code will put the LCD cursor at Pos12
		call	LCD_Line2W	; Line 2 LCD
		movlw	'M'		; M
		call	LCDChar		;
		movlw	'1'		; 1
		call	LCDChar		;
		movlw	'0'		; 0
		call	LCDChar		;
		movlw	'='		; =
		call	LCDChar		;
		movlw	'?'		; ?
		call	SW_Ti		; Wait for data to enter
		movlw	Pos08		; This code will put the LCD cursor at Pos08
		call	LCD_Line1W	; Line 1 LCD
		movf	DIGIT,W		;
		movwf  	m10		;
		call	HEX_Table	; Routine calling data from a DT table
		call	LCDChar		; Write the character data to the LCD
;/************************************\	;
; Minutes 01's				;
;\************************************/	;
		movlw	Pos12		; This code will put the LCD cursor at Pos12
		call	LCD_Line2W	; Line 2 LCD
		movlw	'M'		; M
		call	LCDChar		;
		movlw	'0'		; 0
		call	LCDChar		;
		movlw	'1'		; 1
		call	LCDChar		;
		movlw	'='		; =
		call	LCDChar		;
		movlw	'?'		; ?
		call	SW_Ti		; Wait for data to enter
		movlw	Pos09		; This code will put the LCD cursor at Pos09
		call	LCD_Line1W	; Line 1 LCD
		movf	DIGIT,W		;
		movwf  	m01		;
		call	HEX_Table	; Routine calling data from a DT table
		call	LCDChar		; Write the character data to the LCD
;/************************************\	;
; Finished. 2 Second delay an then return	
;\************************************/	;
TwoSec:		call	Del255		; 8*255ms = 2sec time-out
		call	Del255		; before the time is displayed
		call	Del255		; on LCD
		call	Del255		;		
		call	Del255		;
		call	Del255		;
		call	Del255		;
		call	Del255		;		
		call	LCD_Clr		; Clear
		return			;
;___________________________________________________________________________________________________
SW_Ti		btfss	Time			; This sequence requires the user to enter data. If Nothing is entered,
		goto	$ - 1			; the sequence will loop forever
		movlw	0xFF			;
		movwf	DIGIT			;
SWE_L1		incf	DIGIT,F			; DIGIT is the currently displayed number
		movf	DIGIT,W			; Move the current DIGIT value into W
		sublw	9			; Test if value is not > than 9
		btfss	STATUS,C		; Test for Carry on status
		clrf	DIGIT			; clear DIGIT register
		movf	DIGIT,W			; No, DIGIT value < than 9
		movlw	Pos16			; This code will put the LCD cursor at Pos16
		call	LCD_Line2W		; LCD Line 2
		movf	DIGIT,W			; Variable for individual 7 Segment value
		call	HEX_Table		; Routine calling data from a DT table
		call	LCDChar			; Write the character data to the LCD
		call	Del50			; Make sure the switch was pressed, and not some bogus bug
swe1		btfsc	Time			; switch Open? Wait for switch to be released
		goto	swe1			; no
		call	Del50			; Make sure the switch was pressed, and not some bogus bug
		btfsc	Time			; Switch still Open?
		goto	swe1			; no
		movlw	D'20'			; Yes, Switch still released (open)
		movwf	count			; Move Dec value 20 into COUNT
swe2		btfsc	Time			; switch Open?
		goto	SWE_L1			; No. Switch was pressed again
		call	Del50			; Make sure the switch was pressed, and not some bogus bug
		decfsz  count,F			; Decrease COUNT
		goto	swe2			; Repeat the Loop
		return				; Return Switch check call
;______________________________________________________________________________________________________
;______________________________________________________________________________________________________
HystPB:
		btfss	CHyst		; switch closed? (Logical 1 = On/Pressed)
		return			; No, back to main loop
		call	Del50		; Make sure the switch was pressed, and not some bogus bug
		btfsc	CHyst		; switch closed? (Logical 1 = On/Pressed)
		goto	$ - 1		; No, Keep on cheking until released
		call	Del50		; Make sure the switch was pressed, and not some bogus bug
		btfsc	CHyst		; Check if the switch was released
		goto	$ - 5		; No, Wait 5ms again
;______________________________________________________________________________________________________	
SW_C		btfss	CHyst		; This sequence requires the user to
		goto	$ - 1		; enter data. If Nothing is entered,
		movlw	0xFF		; the sequence will loop forever
		movwf	DIGIT		;
SWC_L1		incf	DIGIT,F         ; DIGIT is the currently displayed number
		movf	DIGIT,W		; Move the current DIGIT value into W
		sublw	9		; Test if value is not > than 6
		btfss   STATUS,C	; Test for Carry on status
		clrf	DIGIT		;
		movf	DIGIT,W		; No, DIGIT value < than 6
		movlw	Pos16		; This code will put the LCD cursor at Pos16
		call	LCD_Line2W	; LCD Line 2
		movf	DIGIT,W		; Variable for individual 7 Segment value
		call	HEX_Table	; Routine calling data from a DT table
		call	LCDChar		; Write the character data to the LCD
		call	Del50		; 50ms Delay
SWC02		btfsc	CHyst	 	; switch Open? Wait for switch to be released
		goto	$ - 1		; no
		call	Del50		; Yes. Switch was released
		btfsc	CHyst		; Switch still Open?
		goto	$ - 4		; no
		movlw	D'30'		; Yes, Switch still released (open)
		movwf	count		; Move Dec value 20 into COUNT
SwCsw1  	btfsc	CHyst		; switch Open?
		goto	SWC_L1		; No. Switch was pressed again
		call	Del50		; Yes. Switch OPEN
		decfsz  count,F		; Decrease COUNT
		goto    $ - 4		; Repeat the Loop
		movf	DIGIT,W		; 
		call	SP_Table	; get the equivalent Hyst for the decimal value
; -------------------------------------	;
; Enter sequence to write the newly	;
; selected hysteresis value into EEPROM	;
; -------------------------------------	;
		clrf	EE_ADR		; Make sure EEPROM is at pos 0	
		movf	HystH,w		; Get new value of HystH into w
		call	WriteEE		; Write into EEPROM
		movf	HystL,w		; Get new value of HystL into w
		call	WriteEE		; Write into EEPROM
		return			; return Switch check call
; -------------------------------------	;
H_05:		movlw	high(Hy_05)	;
		movwf	HystH		;
		movlw	low(Hy_05)	;
		movwf	HystL		;
		return			;
H_10:		movlw	high(Hy_10)	;
		movwf	HystH		;
		movlw	low(Hy_10)	;
		movwf	HystL		;
		return			;
H_15:		movlw	high(Hy_15)	;
		movwf	HystH		;
		movlw	low(Hy_15)	;
		movwf	HystL		;
		return			;
H_20:		movlw	high(Hy_20)	;
		movwf	HystH		;
		movlw	low(Hy_20)	;
		movwf	HystL		;
		return			;
H_25:		movlw	high(Hy_25)	;
		movwf	HystH		;
		movlw	low(Hy_25)	;
		movwf	HystL		;
		return			;
H_30:		movlw	high(Hy_30)	;
		movwf	HystH		;
		movlw	low(Hy_30)	;
		movwf	HystL		;
		return			;
H_35:		movlw	high(Hy_35)	;
		movwf	HystH		;
		movlw	low(Hy_35)	;
		movwf	HystL		;
		return			;
H_40:		movlw	high(Hy_40)	;
		movwf	HystH		;
		movlw	low(Hy_40)	;
		movwf	HystL		;
		return			;
H_45:		movlw	high(Hy_45)	;
		movwf	HystH		;
		movlw	low(Hy_45)	;
		movwf	HystL		;
		return			;
H_50:		movlw	high(Hy_50)	;
		movwf	HystH		;
		movlw	low(Hy_50)	;
		movwf	HystL		;
		return			;


;______________________________________________________________________________________________________	
;EE_WR			EEPROM DATA WRITE SEQUENCE				;
EE_WR		

		call	BANK3 		; Select Bank of EECON1				|B3
		btfsc	EECON1,WR	; Wait for write				|B3
		goto	$-1		; to complete					|
		call	BANK0		;						|B0
		movf	EE_ADR,W 	;						|B0
		call	BANK2		; (0x10C) Select Bank of EEADR 			|B2
		movwf	EEADR		; Data Memory Address to write			|B2
		call	BANK0		;						|B0
		movf	W_DATA,W	; Register containing data to be stored		|B0
		call	BANK2		; (0x10D)					|B2
		movwf	EEDATA		; Data Memory Value to write			|B2
		bsf	STATUS,RP0	;
		bcf	EECON1,EEPGD	; Point to DATA  memory				|B3
		bsf	EECON1,WREN	; Enable writes					|B3
		bcf	INTCON,GIE	; Disable INTs.					|All
		movlw	H'55'		;						|
		movwf	EECON2		; (0x18C) Write 55h				|B3
		movlw	H'AA'		; 						|
		movwf	EECON2		; (0x18C) Write AAh				|B3
		bsf	EECON1,WR	; Set WR bit to begin write			|B3
		bsf	INTCON,GIE	; Enable INTs.					|All
		bcf	EECON1,WREN	; Disable writes				|B3
		call	BANK0		;
		btfss	PIR2,EEIF	; Wait for interrupt flag to be set
		goto	$-1		; Current line -1
		bcf	PIR2,EEIF	; Clear the EEWRITE interrupt flag
		return			; return to caller				|

; *************************************************************************************	;
;	EE_RD			EEPROM DATA READ SEQUENCE				;
; *************************************************************************************	;
EE_RD						;
		movf	EE_ADR,W		;
		call	BANK2			;
		movwf	EEADR			; Data Memory Address to read
		call	BANK3			;
		bcf	EECON1,EEPGD		; Point to Data memory
		bsf	EECON1,RD		; EE Read
		call	BANK2			;
		movf	EEDATA,W 		; W = EEDATA
		call	BANK0			;
		movwf	R_DATA			; Move W into EE_BYTE
		return				; Value now in W, to be used

;----------------------------------------------	;
; WRITE	routine to write data			;
; into EEPROM memory Hours			;
;----------------------------------------------	;
WriteEE:					;	
		movwf	W_DATA			; Writes Hous data into EEPROM
		call	EE_WR			; Write routine for EE_Addr
		incf	EE_ADR,f		; increment EE_Addr with 1
		return				; return to caller
;----------------------------------------------	;
;ReadEE:
;		movlw	0x02		; Move required EE_Adr into W
;		movwf	EE_ADR		; EERdAdr register for EE_Read values
;		call	EE_RD		;
;		nop	
;		return			;

;/************************************\	;
;	BANK0				;
;\************************************/	;
					;
BANK0:		bcf	STATUS,RP0	; select register bank 0
		bcf	STATUS,RP1	;
		bcf	STATUS,IRP	;
		return			;
					;
;/************************************\	;
;	BANK1				;
;\************************************/	;
					;
BANK1:		bsf	STATUS,RP0	; select register bank 1
		bcf	STATUS,RP1	;
		bcf	STATUS,IRP	;
		return			;
					;
;/************************************\	;
;	BANK2				;
;\************************************/	;
					;
BANK2:		bcf	STATUS,RP0	; select register bank 2
		bsf	STATUS,RP1	;
		bsf	STATUS,IRP	;
		return			;
					;
;/************************************\	;
;	BANK3				;
;\************************************/	;
					;
BANK3:		bsf	STATUS,RP0	; select register bank 3
		bsf	STATUS,RP1	;
		bsf	STATUS,IRP	;
		return			;

;___________________________________________________________________________________________________
D160us	
		clrf	count1		;	
		bsf	count1, 5	;  Delay 160 usecs
		bsf	count1, 4	;
		decfsz	count1, f	;
		goto	$ - 1		;
		return			;
;___________________________________________________________________________________________________
; _____	Delay routines ________________________________________________	;
;	Some delay routines range from 1,5,20,50 and 100ms delay loops 	;
;___________________________________________________________________________________________________
Del255		movlw	0xff		; delay 255 mS
		goto	d0		;
Del200		movlw	d'200'		; delay 200mS
		goto	d0		;
Del100		movlw	d'100'		; delay 100mS
		goto	d0		;
Del50		movlw	d'50'		; delay 50mS
		goto	d0		;
Del20		movlw	d'20'		; delay 20mS
		goto	d0		;
Del05		movlw	0x05		; delay 5.000 ms (4 MHz clock)
		goto	d0		;
Del01		movlw	0x01		; delay 1.000 ms (4 MHz clock)
d0		movwf	count1		;
d1		movlw	0xC7		; delay 1mS
		movwf	counta		;
		movlw	0x01		;
		movwf	countb		;
Del_0		decfsz	counta,f	;
		goto	$+2		;
		decfsz	countb,f	;
		goto	Del_0		;
		decfsz	count1,f	;
		goto	d1		;
		retlw	0x00		;

;______________________________________________________________________	;
;	Interrupt service routine
;	Save register data
;______________________________________________________________________	;
ISR		movwf	w_temp		; Save W register
		swapf	STATUS,W	; Swap status to be saved into W
		movwf	s_temp		; Save STATUS register
;______ Set timer1 data values to countdown from ______________________	;
		bcf	PIR1,TMR1IF	; Reset Timer overflow flag
		clrf	TMR1L		; Clear TMR1L (lo byte) to prevent roll over
		movlw	high(Val)	;
		movwf	TMR1H		; write value into hi_byte timer register
		movlw	low(Val)	;
		movwf	TMR1L		; write value into lo_byte timer register
		clrf	PIR1		;
;______ Place required code here ______	;
		incf	Qs,F		; increment 1/4 seconds with 1
		movf	Qs,W		; move Qs into W to test if it's > than 1
		sublw	1		; subract 1
		btfsc	STATUS,C	; test if the Carry bit is set
		goto	$ + 2		;
		goto	$ + 3		;
		bsf	Show		;
		goto	IntEnd		; N
		clrf	Qs		; Y
		bcf	Show		;
;______ Seconds _______________________	;
		incf	sec,f		; increment seconds with 1
		movf	sec,w		; now test if sec > 60
		sublw	0x3c		; subtract 60 from sec
		btfss	STATUS,Z	; test if the ZERO bit is set
		goto	RunT		; 0
		clrf	sec		; 1
;______ Minutes 01_____________________	;
		incf	m01,f		; increment mins 01 with 1
		movf	m01,w		; now test if min > 60
		sublw	0x0A		; subtract dec 10 to test if overflowing
		btfss	STATUS,Z	; test if the ZERO bit is set
		goto	RunT		; 0
		clrf	m01		; 1
;______ Minutes 10_____________________	;
		incf	m10,f		; increment min 10 with 1
		movf	m10,w		; now test if min > 6
		sublw	0x06		; subtract dec 10 to test if overflowing
		btfss	STATUS,Z	; test if the ZERO bit is set
		goto	RunT		; 0
		clrf	m10		; 1
;______ Hours 01_______________________	;
		movf	h10,w		; move hour to w to test	
		sublw	0x02		; Test if h01 > 2
		btfss	STATUS,Z	; test if the Carry bit is set
		goto	IncHr		; Y H10 => 2
		incf	h01,f		; increment h01 with 1
		movf	h01,w		; move hour to w to test
		sublw	0x04		; Test if h01 > 4
		btfss	STATUS,Z	; test if the Carry bit is set
		goto	IntEnd		; N
		goto	ClrHr		; Y H01 > 4 (Hr = 24)
IncHr		incf	h01,f		; increment h01 with 1
		movf	h01,w		; move hour to w to test
		sublw	0x0a		; Test if h01 > 9
		btfss	STATUS,Z	; test if the Carry bit is set
		goto	IntEnd		; Y
		clrf	h01		; N
;______ Hours 10_______________________	;
		incf	h10,f		; increment h10 with 1
		goto	IntEnd		;
;______ Display time __________________	;
RunT:
		btfss	QRel		;
		goto	IntEnd		;
;______ Seconds01 _____________		;
		incf	Rsec01,f	; Increment the sec01 with 1
		movf	Rsec01,w	; now test if sec > 10
		sublw	.10		; subtract 10 from sec
		btfss	STATUS,Z	; test if the ZERO bit is set
		goto	IntEnd		; 0
		clrf	Rsec01		; 1
;______ Seconds10 _____________		;
		incf	Rsec10,f	; Increment sec10 with 1
		movf	Rsec10,w	; now test if sec > 6
		sublw	.06		; subtract 10 from sec
		btfss	STATUS,Z	; test if the ZERO bit is set
		goto	IntEnd		; 0
		clrf	Rsec10		; 1
;______ Minutes 01_____________		;
		incf	Rmin01,f	;increment mins 01 with 1
		movf	Rmin01,w	;now test if min > 60
		sublw	.10		;subtract dec 10 to test if overflowing
		btfss	STATUS,Z	;test if the ZERO bit is set
		goto	IntEnd		;0
		clrf	Rmin01		;1
;______ Minutes 10_____________		;
		incf	Rmin10,f	;increment min 10 with 1
		movf	Rmin10,w	;now test if min > 6
		sublw	.06		;subtract dec 10 to test if overflowing
		btfss	STATUS,Z	;test if the ZERO bit is set
		goto	IntEnd		;0
		clrf	Rmin10		;1
		goto	IntEnd		; 0
;______ Hours 10_______________________	;
ClrHr:		clrf	h01		;
		clrf	h10		;
		clrf	Rsec01		;
		clrf	Rsec10		;
		clrf	Rmin01		;
		clrf	Rmin10		;
		clrf	Rcount01	;
		clrf	Rcount10	;
;______ Restore registers data ________	;
IntEnd:					; Interrupt routine finished. Restore original 
		swapf	s_temp,W	; register values.
		movwf	STATUS		; Restore STATUS register - restores bank
		swapf	w_temp,F	;
		swapf	w_temp,W	; Restore W register
;______ Return to interrupt caller ____	;
		retfie			; Return interrupt call
;___________________________________________________________________________________________________
		end				;
;___________________________________________________________________________________________________

 DOCUMENT CONTROL AND CHANGES:

; Description:
; This design measures temperature using a silicon diode 1N4148
; as primary measurement element.
; The 1N4148 diode has an average of about 2mV/C volt-drop across the diode
; in a negative coefficient. The Mv signal from the diode is scaled and amplified
; via a LM385 operational amplifier, with offset voltage divider on the non-inverting
; input and a gain circuit of about 10 on the inverting side. The LM358 and diode is fed
; from a 7808 (8V) voltage regulator to allow a full swing of 0..5V from the op amp output.
;
; The 0..5V signal gives a 0..1023 10 bit resolution on the PIC ADRESH and ADRESL registers.
; This result is multiplied with 1000 to give a 24-bit value of 1023000.
;
; To scale the measurement to 0-100C, the result from the multiplication must be divided by
; 1023 (3FF). This will give a max value of 1000, where the last value is the decimal point value.
;
; To scale the temperature measurement between 0 and 50C, the full scale A/D value is divided
; with 50 and that gives a factor of 20.46. Because the PIC cannot do decimal point calculation
; the measured signal is multiplied with 1000, and the 'denominator' is multiplied with 100.
; This is a constant value, hence it's hard-coded into the software (20.46 * 1000 = 2046 = 0x07FE)
; The result is a value ranging from 0 to 500. By manually placing the decimal point on the LCD 
; display routine, I get a result of 00.0 to 50.0C
;
; Ver 1.1
; Changed the V+Ref from external reference to internal Vss
;_________________________________________________________________________________________________
; Credit where it is due:
; Some of the code within this file came from other sources. I did not in all cases put the original author's
; name next to the code because it was used over and over again in many other applications of mine, and the
; names have disappeared with the numerous copying and pasting. It's not my intension to mass produce this 
; application and to make profit from this. I did it for my own personal use. I apologize to anybody that might
; recognize some parts of this code to be his/hers. 
;_________________________________________________________________________________________________




